@@ -5,8 +5,7 @@
 - Decoders: ASCIIHexDecode, ASCII85Decode, FlateDecode,
 LZWDecode, RunLengthDecode.
 - Encoders: ASCIIHexDecode, ASCII85Decode, FlateDecode, RunLengthDecode.
-- Predictors: PNG (all), TIFF (8-bit only).
-
+- Decode predictors: PNG (all), TIFF (8-bit only).
 *)
 
 (** {2 Types} *)
@@ -34,11 +33,10 @@ exception Couldn'tDecodeStream of string
 (** CamlPDF doesn't support this encoding or its predictor. *)
 exception DecodeNotSupported of string
 
-
 (** {2 Encoding} *)
 
 (** Encode a PDF stream with an encoding. The only predictor supported is PNGUp. *)
-val encode_pdfstream : Pdf.t -> encoding -> ?predictor:predictor -> ?predictor_columns:int -> Pdf.pdfobject -> unit
+val encode_pdfstream : Pdf.t -> encoding -> ?only_if_smaller:bool -> ?predictor:predictor -> ?predictor_columns:int -> Pdf.pdfobject -> unit
 
 (** {2 Decoding} *)
 
@@ -79,3 +77,5 @@ val debug : bool ref
 
 (* Inter-module recursion. *)
 val string_of_pdf : (Pdf.pdfobject -> string) ref
+
+val encode_predictor : int -> int -> int -> int -> Pdfio.bytes -> Pdfio.bytes
