@@ -8,7 +8,7 @@ of PDFs. Equal names imply equal PDFs (for efficiency). [ranges] is a list of
 page ranges. A page range is a list of pages to select, in order. For example
 [[1]] or [[2; 2; 2; 5; 6; 7]].  *)
 val merge_pdfs :
-  bool -> bool -> string list -> Pdf.t list -> int list list -> Pdf.t
+  bool -> bool -> ?struct_hierarchy:bool -> string list -> Pdf.t list -> int list list -> Pdf.t
 
 (** Remove duplicate fonts from a PDF. For example, if it was created by merging
 several documents from the same source. *)
