@@ -69,7 +69,7 @@ type t =
   | Op_K of float * float * float * float (* Set CMYK stroking *)
   | Op_k of float * float * float * float (* Set CMYK nonstroking *)
   | Op_sh of string (* Shading pattern *)
-  | InlineImage of (Pdf.pdfobject * bytes) (* Inline image dictionary/data *)
+  | InlineImage of (Pdf.pdfobject * Pdf.pdfobject option * bytes) (* Inline image dictionary, previous decodeparams if any, data *)
   | Op_Do of string (* Introduce an XObject *)
   | Op_MP of string (* Marked content point *)
   | Op_DP of string * Pdf.pdfobject (* same with property list *)
@@ -84,7 +84,7 @@ type lexeme =
   | Op of string
   | Obj of Pdfgenlex.t
   | PdfObj of Pdf.pdfobject
-  | LexInlineImage of (Pdf.pdfobject * bytes)
+  | LexInlineImage of (Pdf.pdfobject * Pdf.pdfobject option * bytes)
   | LexComment
   
 (* Lexing *)
@@ -205,7 +205,7 @@ let lexemes_of_op f = function
       f (Obj (Pdfgenlex.LexReal c)); f (Obj (Pdfgenlex.LexReal m));
       f (Obj (Pdfgenlex.LexReal y)); f (Obj (Pdfgenlex.LexReal k)); f (Op "k")
   | Op_sh s -> f (Obj (Pdfgenlex.LexName s)); f (Op "sh")
-  | InlineImage (dict, data) -> f (LexInlineImage (dict, data))
+  | InlineImage (dict, dp, data) -> f (LexInlineImage (dict, dp, data))
   | Op_Do s -> f (Obj (Pdfgenlex.LexName s)); f (Op "Do")
   | Op_MP s -> f (Obj (Pdfgenlex.LexName s)); f (Op "MP")
   | Op_DP (s, obj) ->
@@ -243,13 +243,33 @@ let string_of_lexeme = function
   | Obj o -> Pdfread.string_of_lexeme o 
   | Op op -> op
   | PdfObj obj -> Pdfwrite.string_of_pdf obj
-  | LexInlineImage (dict, data) ->
+  | LexInlineImage (dict, dp, data) ->
       (* Compress if no compression *)
       let dict, data =
         match Pdf.lookup_direct_orelse (Pdf.empty ()) "/F" "/Filter" dict with
         | None | Some (Pdf.Array []) ->
-            Pdf.add_dict_entry dict "/F" (Pdf.Name "/Fl"),
-            Pdfcodec.encode_flate data
+            (* Was there previously a predictor? *)
+            let dict, data =
+              match dp with
+              | None -> dict, data
+              | Some d ->
+                  let colours = match Pdf.lookup_direct (Pdf.empty ()) "/Colors" d with Some (Pdf.Integer x) -> x | _ -> 1 in
+                  let bpc = match Pdf.lookup_direct_orelse (Pdf.empty ()) "/BPC" "/BitsPerComponent" d with Some (Pdf.Integer x) -> x | _ -> 8 in
+                  let columns = match Pdf.lookup_direct (Pdf.empty ()) "/Columns" d with Some (Pdf.Integer x) -> x | _ -> 1 in
+                    match colours, bpc with
+                    | 3, 8 ->
+                        let data = Pdfcodec.encode_predictor 11 colours bpc columns data in
+                        let dict =
+                          Pdf.add_dict_entry dict "/DP"
+                            (Pdf.Dictionary [("/Colors", Pdf.Integer colours);
+                                             ("/Columns", Pdf.Integer columns);
+                                             ("/Predictor", Pdf.Integer 11)])
+                        in
+                          dict, data
+                    | _, _ -> dict, data
+            in
+              Pdf.add_dict_entry dict "/F" (Pdf.Name "/Fl"),
+              Pdfcodec.encode_flate data
         | _ -> dict, data
       in
       let dict = Pdf.add_dict_entry dict "/L" (Pdf.Integer (bytes_size data)) in
@@ -329,7 +349,7 @@ let lex_number i =
 (* Lex and parse a dictionary to a Pdf.pdfobject. This constitutes a single
 lexeme in terms of this module. *)
 let get_dictionary i =
-  PdfObj (snd (Pdfread.parse (Pdfread.lex_dictionary i)))
+  PdfObj (snd (Pdfread.parse (Pdfread.lex_dictionary false i)))
 
 (* Given a colourspace and the number of bits per component, give the number of
 bytes per pixel in the stored image data. *)
@@ -374,7 +394,7 @@ let lex_inline_image pdf resources i =
   if !debug then Pdfe.log (Printf.sprintf "lex_inline_image at %i\n" (i.pos_in ()));
   try
   let dict =
-    let lexemes = Pdfread.lex_dictionary i in
+    let lexemes = Pdfread.lex_dictionary true i in
       snd
         (Pdfread.parse
           ([Pdfgenlex.LexLeftDict] @ lexemes @ [Pdfgenlex.LexRightDict]))
@@ -529,7 +549,8 @@ let lex_inline_image pdf resources i =
                         dict
                         ["/Filter"; "/F"; "/DecodeParms"; "/DP"] 
                 in
-                  dict', data
+                let dp = Pdf.lookup_direct_orelse (Pdf.empty ()) "/DP" "/DecodeParms" dict in
+                  dict', dp, data
     | _ ->
         Pdfe.log "Did not recognise beginning of inline image ID\n";
         nocontent i
@@ -590,12 +611,12 @@ let lex_next pdf resources i =
     | Couldn'tHandleContent ->
         raise (Pdf.PDFError "Malformed page content")
 
-let print_lexeme = function
+(*let print_lexeme = function
   | Obj p -> Pdfread.print_lexeme p
   | Op s -> print_string s; print_newline ()
   | PdfObj p -> print_string "PDF OBJECT\n"
   | LexInlineImage _ -> print_string "INLINE IMAGE\n"
-  | LexComment -> print_string "COMMENT\n"
+  | LexComment -> print_string "COMMENT\n"*)
 
 (* Lex a graphics stream *)
 let lex_stream pdf resources i =
@@ -890,12 +911,12 @@ let stream_of_lexemes (oplists : lexeme list list) =
             done;
             s
 
-let print_stream s =
+(*let print_stream s =
   if bytes_size s > 0 then 
     for x = 0 to bytes_size s - 1 do
       Printf.printf "%c" (char_of_int (bget s x))
     done;
-  print_newline ()
+  print_newline ()*)
     
 (* Make a stream from a list of operators. *)
 let stream_of_ops ops =
