@@ -228,8 +228,20 @@ let pages_of_pagetree_quick pdf =
 let endpage pdf =
   pages_of_pagetree_quick pdf
 
+let endpage_fast pdf =
+  let document_catalog =
+    try Pdf.lookup_obj pdf pdf.Pdf.root with
+      Not_found -> raise (Pdf.PDFError "/Root entry is incorrect")
+  in 
+  let pages =
+    Pdf.lookup_fail "No or malformed /Pages" pdf "/Pages" document_catalog
+  in
+    match Pdf.lookup_direct pdf "/Count" pages with
+    | Some (Pdf.Integer i) -> i
+    | _ -> 0
+
 let _ =
-  Pdfread.endpage := endpage
+  Pdfread.endpage := endpage_fast
 
 (* Make a collection of pages capable of being merged -- in other words rename
 their resources so as not to clash. *)
@@ -298,7 +310,7 @@ let change_operator pdf lookup lookup_option seqnum = function
             Pdfe.log "Warning: Missing Op_BDC /Properties entry\n";
             Pdfops.Op_BDC (n, Pdf.Name p)
       end
-  | Pdfops.InlineImage (dict, bytes) ->
+  | Pdfops.InlineImage (dict, dp, bytes) ->
       (* Replace any indirect "/CS" or "/ColorSpace" with a new "/CS" *)
       let dict' =
         match Pdf.lookup_direct_orelse pdf "/CS" "/ColorSpace" dict with
@@ -317,7 +329,7 @@ let change_operator pdf lookup lookup_option seqnum = function
               (Pdf.Name (lookup "/ColorSpace" seqnum n))
         | _ -> dict
       in
-        Pdfops.InlineImage (dict', bytes)
+        Pdfops.InlineImage (dict', dp, bytes)
   | x -> x
 
 (* Only for use with twoup now. FIXME: Can blow up shared content streams. Needs
@@ -327,7 +339,7 @@ let renumber_pages pdf pages =
   | [] -> []
   | pages ->
       let changes = make_changes pdf pages in
-        let rec lookup_option dictname page oldkey =
+        let lookup_option dictname page oldkey =
           tryfind changes (dictname, page, oldkey)
         and lookup dictname page oldkey =
           try
@@ -395,15 +407,15 @@ let rec pagetree objnumsource pages parent =
         in let right_tree = pagetree objnumsource right this_num in
           Br (this, left_tree, right_tree, parent, this_num)
 
-let pagetree_flat objnumsource pages parent =
-  Lf (pages, parent, objnumsource ())
+(*let pagetree_flat objnumsource pages parent =
+  Lf (pages, parent, objnumsource ())*)
 
 (* Version for pdf_of_pages where we are using the same object numbers *)
 type ptree_objnumbers =
   | OLf of int list * int * int (* object numbers, parent, object number of this leaf *)
   | OBr of int list * ptree_objnumbers * ptree_objnumbers * int * int (* object numbers, left, right, parent, object number of this branch *)
 
-let rec print_ptree = function
+(*let rec print_ptree = function
   | OLf (is, parent, objnumleaf) ->
       Printf.printf "OLf with object numbers ";
       print_ints is;
@@ -415,7 +427,7 @@ let rec print_ptree = function
       Printf.printf "***LEFTS\n";
       print_ptree l;
       Printf.printf "***RIGHTS\n";
-      print_ptree r
+      print_ptree r*)
 
 let rec pagetree_with_objnumbers toplevel old_pagetree_root_num objnumsource objnumbers parent =
   if length objnumbers < 10 then
@@ -626,9 +638,9 @@ let change_pages_find_matrix dest mattable refnumstable =
               Hashtbl.find mattable pagenumber
           with
             _ ->
-              Pdfe.log (Printf.sprintf
+              (*Pdfe.log (Printf.sprintf
                 "page not found for bookmark or annotation dest:%s\n"
-                (Pdfwrite.string_of_pdf (Pdfdest.pdfobject_of_destination dest)));
+                (Pdfwrite.string_of_pdf (Pdfdest.pdfobject_of_destination dest)));*)
               Pdftransform.i_matrix
           end
       | _ -> Pdftransform.i_matrix
@@ -867,14 +879,10 @@ let pdf_of_pages_build_pagetree thetree objnumbers pdf =
 (* pdf_of_pages, if it has duplicates in the range, will produce duplicate
 items in the page tree, pointing to the same page object. This is bad for
 two reasons:
-
    a) Adobe Reader is broken and crashes in this case
-
    b) In any event, duplicate references make further document changes
    confusing for most programs.  So, we duplicate the actual page objects, and
-   do the minimal renumbering.
-
-*)
+   do the minimal renumbering.  *)
 
 (* Given a number n, of a page node, copy it to a new object, and rewrite all
 but the first instance in the page tree to that new number. *)
@@ -969,9 +977,44 @@ let pdf_of_pages ?(retain_numbering = false) basepdf range =
     let refnums = Pdf.page_reference_numbers basepdf in
     let fastrefnums = hashtable_of_dictionary (combine refnums (indx refnums)) in
     let table = hashset_of_list range in
+    let firstindex = ref ~-1 in
+    let firstindexlevel = ref ~-1 in
+    let index = ref ~-1 in
+    let marks = Pdfmarks.read_bookmarks basepdf in
+    let r =
       option_map
-        (function m -> if Hashtbl.mem table (pagenumber_of_target ~fastrefnums basepdf m.Pdfmarks.target) then Some m else None)
-        (Pdfmarks.read_bookmarks basepdf)
+        (function m ->
+          index += 1;
+          if Hashtbl.mem table (pagenumber_of_target ~fastrefnums basepdf m.Pdfmarks.target) then
+            begin
+              (*Printf.printf "Keeping index %i: %s\n" !index m.Pdfmarks.text;*)
+              if !firstindex = -1 then
+                begin
+                  firstindex := !index;
+                  firstindexlevel := m.Pdfmarks.level
+                end;
+              Some m
+            end
+          else
+            None)
+        marks
+    in
+      (*Printf.printf "Index of first saved bookmark: %i, at level %i\n" !firstindex !firstindexlevel;*)
+      (* Find the last bookmark at firstindexlevel - 1, firstindexlevel -2 .... 0 and add them as prefixes *) 
+      firstindexlevel -= 1;
+      let got = ref [] in
+      for n = !firstindex downto 0 do
+        let mark = List.nth marks n in
+        (*Printf.printf "n = %i: %s\n" n mark.Pdfmarks.text;*)
+        if !firstindexlevel > -1 then
+          if mark.Pdfmarks.level = !firstindexlevel then
+            begin
+              (*Printf.printf "got a mark at level %i\n" !firstindexlevel;*)
+              got := mark::!got;
+              firstindexlevel -= 1
+            end
+      done;
+      !got @ r
   in
     let pdf = Pdf.empty () in
       Pdf.objiter (fun k v -> ignore (Pdf.addobj_given_num pdf (k, v))) basepdf;
@@ -1183,7 +1226,7 @@ let prefix_operator pdf p = function
   | Pdfops.Op_Do x -> Pdfops.Op_Do (addp p x)
   | Pdfops.Op_DP (n, Pdf.Name x) -> Pdfops.Op_DP (n, Pdf.Name (addp p x))
   | Pdfops.Op_BDC (n, Pdf.Name x) -> Pdfops.Op_BDC (n, Pdf.Name (addp p x))
-  | Pdfops.InlineImage (dict, bytes) ->
+  | Pdfops.InlineImage (dict, dp, bytes) ->
       (* Replace any indirect "/CS" or "/ColorSpace" with a new "/CS" *)
       let dict' =
         match Pdf.lookup_direct_orelse pdf "/CS" "/ColorSpace" dict with
@@ -1197,7 +1240,7 @@ let prefix_operator pdf p = function
               (Pdf.Name (addp p n))
         | _ -> dict
       in
-        Pdfops.InlineImage (dict', bytes)
+        Pdfops.InlineImage (dict', dp, bytes)
   | x -> x
 
 let change_resources pdf prefix resources =
